SUBROUTINE rfunc(coefs,nparams,vnodes,npoints,betashock,nstates,P,vstart,vend,Rvec)

USE model_parameters
USE wrapper_coef

IMPLICIT NONE

INTEGER, INTENT(IN) :: nparams, npoints, nstates

DOUBLE PRECISION, DIMENSION(nparams), INTENT(IN) :: coefs
DOUBLE PRECISION, DIMENSION(npoints), INTENT(IN) :: vnodes
DOUBLE PRECISION, DIMENSION(nstates), INTENT(IN) :: betashock
DOUBLE PRECISION, DIMENSION(nstates,nstates), INTENT(IN) :: P
DOUBLE PRECISION, INTENT(IN) :: vstart, vend

DOUBLE PRECISION, DIMENSION(npoints*nstates*3), INTENT(OUT) :: Rvec

INTEGER :: order, state, nextstate, status, inode, posindx
DOUBLE PRECISION, ALLOCATABLE, TARGET, DIMENSION(:) :: coef_c_forstate,coef_c_fornextstate, coef_x1_forstate, coef_x1_fornextstate, coef_x2_forstate, coef_x2_fornextstate

DOUBLE PRECISION :: y, c, l, mc, x1, x2, w, pi, pistar, v, r, z, betalevel, betalevel_prime
DOUBLE PRECISION :: y_prime, c_prime, l_prime, mc_prime, x1_prime, x2_prime, w_prime, pi_prime, pistar_prime,v_prime, r_prime, z_prime
DOUBLE PRECISION :: expectation_term_c, this_term_c
DOUBLE PRECISION :: expectation_term_x1, this_term_x1
DOUBLE PRECISION :: expectation_term_x2, this_term_x2

DOUBLE PRECISION, DIMENSION(1) :: pivec, residvec
DOUBLE PRECISION :: resid
INTEGER :: n, info

EXTERNAL :: chebypol, hfunc, solve_current_wrapper


order = nparams/3/nstates-1

ALLOCATE( coef_c_forstate(order+1), coef_c_fornextstate(order+1), coef_x1_forstate(order+1), coef_x1_fornextstate(order+1), coef_x2_forstate(order+1), coef_x2_fornextstate(order+1), STAT=status)

IF (status .NE. 0) THEN
   WRITE(*, *) 
   WRITE(*, *) '***In Call_....: Could not allocate memory***'
END IF

! assign pointers to variables -- pointers are declared in
! the module wrapper_var_solve_model

posindx = 0
DO state = 1,nstates
    ! unpack the cofficients for the polynomial rules given this state
    coef_c_forstate = coefs((order+1)*(state-1)+1:(order+1)*state)
    coef_x1_forstate = coefs((order+1)*(state-1)+(order+1)*nstates+1:(order+1)*nstates+(order+1)*state)
    coef_x2_forstate = coefs((order+1)*(state-1)+2*(order+1)*nstates+1:2*(order+1)*nstates+(order+1)*state)

    DO inode = 1,npoints
       posindx = posindx+1

       !CALL hybrd1(ssnorisk,n,kssvec,resid,0.0000000001d0,info)
       !SUBROUTINE ssrisk
       betalevel = betap*exp(betashock(state));

       ! set up variables for call to solve_current
       vlag_copy = vnodes(inode)
       betalevel_copy = betalevel
       n = 1
       pivec(1) = piss

       coef_c_forstate_ptr => coef_c_forstate
       coef_x1_forstate_ptr => coef_x1_forstate
       coef_x2_forstate_ptr => coef_x2_forstate

       CALL hybrd1(solve_current_wrapper,n,pivec,residvec,0.0000000001d0,info)
       pi = pivec(1)
       CALL solve_current(pi,vnodes(inode),betalevel,resid, pistar, v, x2, x1, mc, c, r, y, l, w)


       ! compute exptectation terms
       ! initialize expectations than run through all possible next states
       expectation_term_c = 0.0d0
       expectation_term_x1 = 0.0d0
       expectation_term_x2 = 0.0d0

       NULLIFY(coef_c_forstate_ptr,coef_x1_forstate_ptr,coef_x2_forstate_ptr)
       coef_c_forstate_ptr => coef_c_fornextstate
       coef_x1_forstate_ptr => coef_x1_fornextstate
       coef_x2_forstate_ptr => coef_x2_fornextstate


       DO nextstate = 1,nstates
            ! unpack the polynomial coefficients for the next state
            coef_c_fornextstate = coefs((order+1)*(nextstate-1)+1:(order+1)*nextstate)
            coef_x1_fornextstate = coefs((order+1)*(nextstate-1)+(order+1)*nstates+1:(order+1)*nstates+(order+1)*nextstate)
            coef_x2_fornextstate = coefs((order+1)*(nextstate-1)+2*(order+1)*nstates+1:2*(order+1)*nstates+(order+1)*nextstate)
            betalevel_prime = betap*exp(betashock(nextstate));
            vlag_copy = v
            betalevel_copy = betalevel_prime
            n = 1
            pivec(1) = piss

            CALL hybrd1(solve_current_wrapper,n,pivec,residvec,0.0000000001d0,info)
            pi_prime = pivec(1);
            CALL solve_current(pi_prime, v, betalevel_prime, resid, pistar_prime, v_prime, x2_prime, x1_prime, mc_prime, c_prime, r_prime, y_prime, l_prime, w_prime)




            ! contributions of current state to expectation terms
            this_term_c = betalevel/c_prime*r/pi_prime;
            this_term_x1 = betalevel*pi_prime**(epsilonp)*x1_prime;
            this_term_x2 = betalevel*pi_prime**(epsilonp-1.d0)/pistar_prime*x2_prime;

            ! accumulate the expectations
            expectation_term_c = expectation_term_c+P(state,nextstate)*this_term_c;
            expectation_term_x1 = expectation_term_x1+P(state,nextstate)*this_term_x1;
            expectation_term_x2 = expectation_term_x2+P(state,nextstate)*this_term_x2;
       END DO

       NULLIFY(coef_c_forstate_ptr, coef_x1_forstate_ptr, coef_x2_forstate_ptr)

       ! size residuals for the inter-temporal conditions
       Rvec(posindx) = - c + expectation_term_c**(-1)
       Rvec(npoints*nstates+posindx) = - x1 +1/c*mc*y+thetap*expectation_term_x1
       Rvec(npoints*nstates*2+posindx) = - x2 + pistar*(y/c+ thetap*expectation_term_x2)

    
   END DO
END DO



DEALLOCATE(coef_c_forstate, coef_c_fornextstate, coef_x1_forstate, coef_x1_fornextstate, coef_x2_forstate, coef_x2_fornextstate, STAT=status)

END SUBROUTINE
