PROGRAM runsim_sticky
! solves the sticky price model in the "adventures" paper by orthogonal collocation with
! Chebyshev polynomials

! ************************
! Remember to update the path for saving the results file at the end of the program -- the path is hardwired.
! ************************

USE model_parameters
USE wrapper_vars

IMPLICIT NONE

DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: poly
DOUBLE PRECISION, ALLOCATABLE, TARGET, DIMENSION(:,:) :: Pdat
DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: coefs, coefs_ss, Rvec
DOUBLE PRECISION, ALLOCATABLE, TARGET, DIMENSION(:) :: betashockdat, nodesdat
DOUBLE PRECISION :: Pd, betashockd, nsteps,this_step, markov_span
INTEGER :: status
INTEGER :: info, m, n, indxi
INTEGER :: version

CHARACTER(len=5) :: filename


EXTERNAL :: rfunc, rfunc_wrapper, rfunc_wrapperss
EXTERNAL :: markovappr, markovappr_pf



betap = 0.994d0
!betap = 0.98910

varthetap = 1.d0   ! governs Frisch elasticity of labor supply
psip = 1.d0        ! normalization of hours worked = 1
thetap = 0.9d0   ! average duration of price contracts = 4
epsilonp = 6.0d0    ! governs steady state markup = 20!
phipip = 2.50d0
!phipip = 5.d0
!phiyp = 0.25d0
phiyp = 0.25d0
piss = 1.005d0
!piss = 1.00001d0
rhorp = 0.d0
sharegp = 0.2d0
rhobetap = 0.8d0
sigmabetap = 0.005d0

!if thetap = 0.9 use sigmabetap = 0.005
!if thetap = 0.85 use sigmabetap = 0.0035  (and can push up epsilonp to 10)


! deterministic steady state
pistarss = ((1.d0-thetap*piss**(epsilonp-1.d0))/(1.d0-thetap))**(1.d0/(1.d0-epsilonp))
betass = betap
rss = piss/betap
zss = rss
x2ss = pistarss/(1.d0-sharegp)/(1.d0-thetap*betap*piss**(epsilonp-1.d0))
x1ss = (epsilonp - 1.d0)/epsilonp*x2ss
vss = (1.d0-thetap)*pistarss**(-epsilonp)/(1.d0-thetap*piss**epsilonp)
yss = (x1ss*(1.d0-thetap*betap*piss**epsilonp)/(psip*vss**varthetap))**(1.d0/(1.d0+varthetap))
lss = vss*yss
css = (1.d0-sharegp)*yss
mcss = psip*(vss*yss)**varthetap*css
wss = mcss


filename = 'take8'


order = 6

nstates = 51


!vstart = 1.0d0
!vend = 1.d0+6.d0*(vss-1.d0)
vstart = 1.d0
vend = 1.040350302358555d0

!vstart = 0.95d0*vss
!vend = 1.5d0*vss


nsteps = 1.d0

markov_span = 3.5d0



ALLOCATE(nodesdat(order+1), poly(order+1,1),Pdat(nstates,nstates), &
coefs((order+1)*nstates*3), coefs_ss((order+1)*3), &
betashockdat(nstates),Rvec((order+1)*nstates*3), STAT=status)

nodes => nodesdat
P => Pdat
betashock => betashockdat


CALL chebyroots((order+1),vstart,vend,nodes)

WRITE(*,*) 'nodes'
CALL writevec_screen(nodes,(order+1),1,(order+1))





!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  SOLVE MODEL
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! first guess -- keep K constant at its SS value, regardless of theta
! the zeroth order coefficient is fixed at the SS value
! fix the first order coefficient at a small negative value -- if capital
! is above SS, capital is expected to be lower next period.

! first solve the deterministic model, then apply the solution from the
! deterministic model to the stochastic model


coefs = 0.d0  ! first guess for Chebyshev coefficients

! first guesses for the deterministic model 



coefs_ss(1) = 1.d0/(css*rss*betap)
coefs_ss(2) = 0.d0;

coefs_ss((order+1)+1) = (x1ss-mcss/(1.d0-sharegp))/thetap/betap
coefs_ss((order+1)+2) = 0.1d0;

coefs_ss(2*(order+1)+1) = (x2ss/pistarss-1/(1-sharegp))/thetap/betap
coefs_ss(2*(order+1)+2) = 0.1d0;

WRITE(*,*) 'Coefs'
WRITE(*,*) coefs


Pd = 1.d0
betashockd = 0.d0  ! one state of nature, set at steady state for technology
n = (order+1)*3
m = n
!CALL hybrd1(rfunc_wrapper,n,avec(1:(order+1)*nstates),Rvec(1:(order+1)*nstates),0.000000000001d0,info)

CALL rfunc(coefs_ss,n,nodes,order+1,betashockd,1,Pd,vstart,vend,Rvec(1:(order+1)*3))
WRITE(*,*) 'Residuals For Model at First Guess for SS'
WRITE(*,*) Rvec(1:(order+1)*3)

WRITE(*,*) 'Find Deterministic Solution'
CALL lmdif1(rfunc_wrapperss,m,n,coefs_ss,Rvec(1:(order+1)*3),0.00000000001d0,info)
WRiTE(*,*)

WRITE(*,*) 'Info from optimizer'
WRITE(*,*) info
WRITE(*,*) 'Residuals For RBC Model Witout Capital Constraint After  Call'
WRITE(*,*) Rvec(1:(order+1)*3)

WRITE(*,*) 'coefs_ss'
WRITE(*,*) coefs_ss



! use parameters for the deterministic model
! as first guess for all states

DO indxi =1,nstates
    coefs((indxi-1)*(order+1)+1:(indxi)*(order+1))=coefs_ss(1:order+1)
    coefs((order+1)*(nstates)+(indxi-1)*(order+1)+1:(order+1)*nstates+indxi*(order+1))=coefs_ss((order+1)+1:2*(order+1))
    coefs(2*(order+1)*(nstates)+(indxi-1)*(order+1)+1:2*(order+1)*(nstates)+(indxi)*(order+1))=coefs_ss(2*(order+1)+1:3*(order+1))
END DO

WRITE(*,*) 'full coefs'
WRITE(*,*) coefs


! get markov approximation to AR process


IF (nstates>1) THEN
CALL markovappr(rhobetap,sigmabetap,markov_span,nstates,betashock,P)
WRITE(*,*) P
!CALL rouwenhorst(rhobetap,this_step/nsteps*sigmabetap,markov_span,nstates,betashock,P)
ELSE
P = 1.d0
betashock = 0.d0
END IF

! Check output of rfuncirr
!n = (order+1)*nstates*3
!CALL rfunc(coefs,n,nodes,order+1,betashock,nstates,P,vstart,vend,Rvec)
!WRITE(*,*) 'Residuals For Model at First Guess'
!WRITE(*,*) Rvec

! Solve the system of equations to parameterize the polynomial rules
n = (order+1)*nstates*3
m = n
WRITE(*,*) 'Find Stochastic Solution'
CALL lmdif1(rfunc_wrapper,m,n,coefs,Rvec,0.000000001d0,info)
Write(*,*)

!ALTERNATIVE:
!CALL hybrd1(function,n,avec,Rvec,0.0000000001d0,info)


WRITE(*,*) 'info from optimizer'
WRITE(*,*) info


!WRITE(*,*) 'coefs'
!WRITE(*,*) coefs

WRITE(*,*) 'Residuals For Full Model'
WRITE(*,*) Rvec



CALL writevec(coefs, (order+1)*nstates*3, 1, (order+1)*nstates*3,&
     filename//'_coefs.m')

CALL write_params(filename//'_paramfile.m',betap,varthetap,thetap,psip,epsilonp,phipip,phiyp,piss,rhorp,sharegp,rhobetap,sigmabetap,order,nstates,vstart,vend,markov_span)


DEALLOCATE(nodesdat, poly, Pdat, coefs, coefs_ss, betashockdat, Rvec, STAT=status)


END PROGRAM
